{% extends "horizon/common/_modal_form.html" %}
{% load i18n %}
{% block modal-body-right %}
    <p>"Reset status of <b>'{{ share_group_snapshot_name }}'</b> share group snapshot."</p>
{% endblock %}
