{% extends 'base.html' %}
{% load i18n %}
{% block title %}{% trans "Create Share Group Snapshot" %}{% endblock %}

{% block main %}
    {% include 'project/share_group_snapshots/_create.html' %}
{% endblock %}
