{% extends "horizon/common/_modal_form.html" %}
{% load i18n %}
{% block modal-body-right %}
<h3>{% trans "Description" %}:</h3>
<p>{% trans "Select parameters of share group you want to create. " %}</p>
{% endblock %}
