{% extends "horizon/common/_modal_form.html" %}
{% load i18n %}
{% block modal-body-right %}
  <h3>{% trans "Description" %}:</h3>
  <p>{% trans "The share type defines the characteristics of a share backend." %}
    <br /><br /><strong>Extra specs</strong> field:<br />
    {% trans "One line - one action. Empty strings will be ignored." %}<br />
    {% trans "To add extra-specs use:" %}
    <pre>key=value</pre>
    {% trans "Extra spec 'driver_handles_share_servers' is required and should have boolean value." %}<br />
  </p>
{% endblock %}
