{% load i18n sizeformat parse_date %}

<h3>{% trans "Share Server Overview" %}</h3>
<div class="detail">
  <hr class="header_rule">
  <dl class="dl-horizontal">
    <dt>{% trans "ID" %}</dt>
    <dd>{{ share_server.id }}</dd>
    <dt>{% trans "Status" %}</dt>
    <dd>{{ share_server.status|capfirst }}</dd>
    <dt>{% trans "Host" %}</dt>
    <dd>{{ share_server.host}}</dd>
    <dt>{% trans "Share Network" %}</dt>
    {% if share_server.share_network_id %}
    {% url 'horizon:admin:share_networks:share_network_detail' share_server.share_network_id as share_net_url %}
    <dd><a href="{{ share_net_url }}">{{ share_server.share_network_name }}</a></dd>
    {% else %}
    <dd>{{ share_server.share_network_name }}</dd>
    {% endif %}
    {% if share_server.shares_list %}
    <dt>{% trans "Shares on this server" %}</dt>
    {% for share in share_server.shares_list %}
    {% url 'horizon:admin:shares:detail' share.id as share_url %}
    <dd><a href="{{ share_url }}">{{ share.name_or_id }}</a></dd>
    {% endfor %}
    {% endif %}
</div>

<div class="detail">
  <h4>{% trans "Specs" %}</h4>
  <hr class="header_rule">
  <dl class="dl-horizontal">
  {% for key, value in share_server.backend_details.items %}
    <dt>{{ key }}</dt>
    <dd>{{ value }}</dd>
  {% endfor %}
  </dl>
</div>

